@@ -168,5 +168,44 @@ def bundle_assets(ftype: str, files: list, dest: str):
         with open(dest, "w") as f:
             f.write(contents)
 
+def merge_files_contents(files: list) -> str:
+    contents = ""
+    for file in files:
+        with open(file) as f:
+            contents += f.read()
+    return contents
+
+def minify_static_contents(ftype:str, contents:str):
+    """
+    To minify contents for static files. 
+
+    Params:
+        ftype:str - type, css|js
+        contents:str - the conent to minify
+    """
+    if ftype == "css":
+        return cssmin.cssmin(contents)
+    elif ftype == "js":
+        return jsmin.jsmin(contents)
+    return contents
+
+def write_file(contents:str, dest:str):
+    """
+    To minify contents for static files. 
+
+    Params:
+        contents:str - the conent to minify
+        dest:path - the output file 
+    """
+    if contents:
+        with open(dest, "w") as f:
+            f.write(contents)
+
 def is_external_url(url:str) -> bool:
-    return url.startswith(('http://', 'https://'))
\ No newline at end of file
+    return url.startswith(('http://', 'https://'))
+
+def context_root(context:str):
+  contexts = context.strip("/").split("/")
+  if len(contexts) == 1:
+    return "./"
+  return "/".join([".." for _ in contexts])
\ No newline at end of file
