@@ -33,6 +33,7 @@ NAME = "SSG.py"
 PAGE_FORMAT = (".html", ".md")
 DEFAULT_LAYOUT = "layouts/default.html"
 
+
 # ==============================================================================
 # -------------------------------- SSG.py -------------------------------------
 # ==============================================================================
@@ -42,6 +43,8 @@ HTML_REDIRECT_TEMPLATE = """<meta http-equiv="Refresh" content="0; url='{url}'"
 def print_info(message):
     print("- %s" % message)
 
+cached = {}
+    
 class Generator(object):
 
     default_page_meta = {
@@ -112,7 +115,7 @@ class Generator(object):
         self.site_config.setdefault("static_url", "/static")
         self.base_url = self.site_config.get("base_url")
         self.static_url = self.site_config.get("static_url")
-
+        self._context_root = "./"
         self._data = {}
         self._load_data()
 
@@ -239,15 +242,15 @@ class Generator(object):
 
     def _make_url(self, url, full_url=False):
         """
-        TODO: FIX relative path where inside of subfolder
         """
         if full_url:
             return self.base_url.rstrip("/") + "/" + url.lstrip("/")
-        url = "./" + url.strip("/")
-        if url == "./":
-            url = "../"
-        return url
-        
+        if len(self._context_root.split("..")) > 1:
+            return self._context_root.rstrip("/") + url
+        if "./" + url.strip("/") == "./":
+            return "../"
+        return url 
+
     def _get_page_static_bundle_url(self, page, bundle_name):
         _page = lib.dictdot(page)
         if url := _page.get("__assets.%s" % bundle_name):
@@ -258,7 +261,10 @@ class Generator(object):
         """Returns the static url """
         if url.startswith("//") or url.startswith("http"):
             return url
-        return self.static_url.rstrip("/") + "/" + url.lstrip("/")
+        url = self.static_url.rstrip("/") + "/" + url.lstrip("/")
+        if len(self._context_root.split("..")) > 1:
+            self._context_root.rstrip("/") + url
+        return url
 
     def _get_dest_file_and_url(self, filepath, page_meta={}):
         """ Return tuple of the file destination and url """
@@ -521,8 +527,7 @@ class Generator(object):
         if redirect := _context.get("page.redirect"):
             url = redirect if lib.is_external_url(redirect) else self._get_page_url(redirect, full_url=True)
             content = HTML_REDIRECT_TEMPLATE.format(url=url)
-            with open(dest_file, "w") as fw:
-                fw.write(content)
+            lib.write_file(dest=dest_file, contents=content)
             return
 
 
@@ -564,13 +569,13 @@ class Generator(object):
                         _sff = _ff.replace(self.build_static_dir, '').lstrip("/")
                         _context["page"]["sfc_%s_path" % sfc_o[o]] = _sff
 
-                        with open(_ff, "w") as f:
-                            o_content = sfc_c.get(o)
-                            o_content = o_content.replace('{%STATIC_URL%}', self.static_url.rstrip("/"))
-                            # convert SCSS to css -> <style scss>
-                            if o == 'style' and "scss" in sfc_c["style_props"].strip():
-                                o_content = lib.scss_to_css(o_content)
-                            f.write(o_content)
+                        # write content
+                        o_content = sfc_c.get(o)
+                        o_content = o_content.replace('{%STATIC_URL%}', self.static_url.rstrip("/"))
+                        # convert SCSS to css -> <style scss>
+                        if o == 'style' and "scss" in sfc_c["style_props"].strip():
+                            o_content = lib.scss_to_css(o_content)
+                        lib.write_file(dest=_ff, contents=o_content)
 
                         if o == "script":
                             # "attributes": sfc_c["script_props"]
@@ -610,18 +615,25 @@ class Generator(object):
                 "js": ("assets_bundles.page_js_bundle", "page.__assets.js", "page_js_bundle"),
             }
             for ftype, s in ftypes.items():
-                flist = [os.path.join(self.static_dir, f) for f in self.config.get(s[0]) or []]
+                cached_contents_ = cached.get(s[2]) or ""
+                if not cached_contents_:
+                    if self.config.get(s[0]):
+                        cached[s[2]] = lib.minify_static_contents(ftype=ftype, contents=lib.merge_files_contents([os.path.join(self.static_dir, f) for f in self.config.get(s[0]) or []]))
+                        cached_contents_ = cached[s[2]]
+
                 if _c := _context.get(s[1]):
-                    flist += [os.path.join(self.build_static_dir, f) for f in _c]
-                if flist:
+                    cached_contents_ += lib.minify_static_contents(ftype=ftype, contents=lib.merge_files_contents([os.path.join(self.build_static_dir, f) for f in _c]))
+
+                if cached_contents_:
                     _context["page"]["__assets"][s[2]] = "%s.%s" %  (bundlename_, ftype)
-                    lib.bundle_assets(ftype=ftype, files=flist, dest="%s.%s" % (filepath_, ftype))
-                    
+                    lib.write_file(contents=cached_contents_, dest="%s.%s" %  (filepath_, ftype))
+
+
         # Write file
         if (self.print_info):
             print_info('creating page: %s...' % filepath)
-        with open(dest_file, "w") as fw:
-            fw.write(tpl.render(**_context))
+        self._context_root = lib.context_root(_context.get("page.url"))
+        lib.write_file(dest=dest_file, contents=tpl.render(**_context))
 
     def build(self, print_info=False):
         self.print_info = print_info
